/*
* Fabric3
* Copyright (c) 2009-2011 Metaform Systems
*
* Fabric3 is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as
* published by the Free Software Foundation, either version 3 of
* the License, or (at your option) any later version, with the
* following exception:
*
* Linking this software statically or dynamically with other
* modules is making a combined work based on this software.
* Thus, the terms and conditions of the GNU General Public
* License cover the whole combination.
*
* As a special exception, the copyright holders of this software
* give you permission to link this software with independent
* modules to produce an executable, regardless of the license
* terms of these independent modules, and to copy and distribute
* the resulting executable under terms of your choice, provided
* that you also meet, for each linked independent module, the
* terms and conditions of the license of that module. An
* independent module is a module which is not derived from or
* based on this software. If you modify this software, you may
* extend this exception to your version of the software, but
* you are not obligated to do so. If you do not wish to do so,
* delete this exception statement from your version.
*
* Fabric3 is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty
* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
* See the GNU General Public License for more details.
*
* You should have received a copy of the
* GNU General Public License along with Fabric3.
* If not, see <http://www.gnu.org/licenses/>.
*/
package org.fabric3.binding.activemq.factory;

import java.net.URI;
import java.util.Collections;
import java.util.List;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import org.apache.activemq.ActiveMQConnectionFactory;
import org.apache.activemq.ActiveMQXAConnectionFactory;
import org.apache.activemq.pool.PooledConnectionFactory;
import org.osoa.sca.annotations.Destroy;
import org.osoa.sca.annotations.EagerInit;
import org.osoa.sca.annotations.Init;
import org.osoa.sca.annotations.Property;
import org.osoa.sca.annotations.Reference;

import org.fabric3.binding.jms.spi.runtime.ConnectionFactoryManager;
import org.fabric3.binding.jms.spi.runtime.FactoryRegistrationException;
import org.fabric3.host.runtime.HostInfo;

/**
 * Instantiates connection factories from settings in the runtime system configuration.
 *
 * @version $Rev: 9763 $ $Date: 2011-01-03 00:48:06 +0000 (Mon, 03 Jan 2011) $
 */
@EagerInit
public class ConnectionFactoryInstantiator {
    private ConnectionFactoryManager manager;
    private ConnectionFactoryConfigurationParser parser;
    private String brokerAddress;

    private List<ConnectionFactoryConfiguration> configurations = Collections.emptyList();

    public ConnectionFactoryInstantiator(@Reference ConnectionFactoryManager manager, @Reference HostInfo info) {
        this.manager = manager;
        parser = new ConnectionFactoryConfigurationParser();
        this.brokerAddress = "vm://" + info.getRuntimeName().replace(":", ".");
    }


    @Property(required = false)
    public void setConnectionFactories(XMLStreamReader reader) throws XMLStreamException, InvalidConfigurationException {
        configurations = parser.parse(brokerAddress, reader);
    }

    @Init
    public void init() throws FactoryRegistrationException {
        // initialize and register the connection factories
        for (ConnectionFactoryConfiguration configuration : configurations) {
            URI uri = configuration.getBrokerUri();
            String name = configuration.getName();
            switch (configuration.getType()) {
            case LOCAL:
                ActiveMQConnectionFactory wrapped = new ActiveMQConnectionFactory(uri);
                wrapped.setProperties(configuration.getFactoryProperties());
                PooledConnectionFactory pooledFactory = new PooledConnectionFactory(wrapped);
                manager.register(name, pooledFactory, configuration.getPoolProperties());
                break;
            case XA:
                ActiveMQXAConnectionFactory xaFactory = new ActiveMQXAConnectionFactory(uri);
                xaFactory.setProperties(configuration.getFactoryProperties());
                manager.register(name, xaFactory, configuration.getPoolProperties());
                break;
            }
        }
    }

    @Destroy
    public void destroy() throws FactoryRegistrationException {
        for (ConnectionFactoryConfiguration configuration : configurations) {
            manager.unregister(configuration.getName());
        }
    }

}
